/*
 * $Id: st_handler.h,v 1.18 2003/03/01 17:14:35 jylefort Exp $
 *
 * This file is part of streamtuner.
 *
 * Copyright (c) 2002-2003, Jean-Yves Lefort
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Jean-Yves Lefort nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _ST_HANDLER_H
#define _ST_HANDLER_H

#include <glib.h>
#include <glib-object.h>
#include "st_category.h"
#include "st_handler_field.h"
#include "st_stream.h"

typedef enum
{
  ST_HANDLER_EVENT_RELOAD,
  
  ST_HANDLER_EVENT_STREAM_NEW,
  ST_HANDLER_EVENT_STREAM_FIELD_GET,
  ST_HANDLER_EVENT_STREAM_FIELD_SET,
  ST_HANDLER_EVENT_STREAM_FREE,

  ST_HANDLER_EVENT_STREAM_TUNE_IN,
  ST_HANDLER_EVENT_STREAM_RECORD,
  ST_HANDLER_EVENT_STREAM_BROWSE,

  ST_HANDLER_EVENT_CATEGORY_NEW,
  ST_HANDLER_EVENT_CATEGORY_FREE,

  N_ST_HANDLER_EVENTS
} STHandlerEvent;

typedef struct
{
  gpointer cb;
  gpointer data;
} STHandlerCallback;

typedef struct _STHandler		STHandler;

/*
 * CATEGORY is the category to reload.
 * CATEGORIES is a location to store a GNode of Category objects
 * STREAMS is a location to store a GList of Stream objects
 *
 * Must return false in case of abort (if the abort was caused by an error
 * condition, ERR must be set).
 *
 */
typedef gboolean   (STHandlerReloadCallback)   (STCategory	*category,
						GNode		**categories,
						GList		**streams,
						gpointer	data,
						GError		**err);

typedef STCategory*(STCategoryNewCallback)     (gpointer	data);
typedef void       (STCategoryFreeCallback)    (STCategory	*category,
						gpointer	data);

typedef STStream*  (STStreamNewCallback)       (gpointer	data);
typedef void	   (STStreamFieldGetCallback)  (STStream	*stream,
						STHandlerField	*field,
						const GValue	*value,
						gpointer	data);
typedef void	   (STStreamFieldSetCallback)  (STStream	*stream,
						STHandlerField	*field,
						GValue		*value,
						gpointer	data);
typedef void	   (STStreamFreeCallback)      (STStream	*stream,
						gpointer	data);

/*
 * STREAM is the stream to be acted upon.
 *
 * Must return false in case of abort (if the abort was caused by an error
 * condition, ERR must be set).
 *
 */
typedef gboolean   (STStreamTuneInCallback)    (STStream	*stream,
						gpointer	data,
						GError		**err);
typedef gboolean   (STStreamRecordCallback)    (STStream	*stream,
						gpointer	data,
						GError		**err);
typedef gboolean   (STStreamBrowseCallback)    (STStream	*stream,
						gpointer	data,
						GError		**err);

struct _STHandler
{
  /* constants */

  const char		*name;			/* mandatory */

  const char		*label;			/* mandatory */
  const char		*copyright;
  
  gpointer		pixbuf;			/*
						 * we don't use the
						 * GdkPixbuf type to not
						 * have to include gdk.h here
						 */

  GList			*fields;
  GNode			*stock_categories;

  STHandlerCallback	callbacks[N_ST_HANDLER_EVENTS];

  /* data */

  GNode			*categories;
  GHashTable		*streams;
  
  GList			*expanded_categories;
  STCategory		*selected_category;
  STStream		*selected_stream;
  char			*selected_category_name;
  char			*selected_stream_name;

  int			fields_sort_index;
  int			fields_sort_order;
};

STHandler  *st_handler_new			(const char	*name);
void       st_handler_set_label			(STHandler	*handler,
						 const char	*label);
void       st_handler_set_copyright		(STHandler	*handler,
						 const char	*copyright);
void	   st_handler_set_icon_from_inline	(STHandler	*handler,
						 int		size,
						 const guint8	*data);
gboolean   st_handler_set_icon_from_file	(STHandler	*handler,
						 const char	*filename,
						 GError		**err);
void       st_handler_set_stock_categories	(STHandler	*handler,
						 GNode		*categories);
void       st_handler_add_field			(STHandler	*handler,
						 STHandlerField	*field);
void       st_handler_bind			(STHandler	*handler,
						 STHandlerEvent	event,
						 gpointer	cb,
						 gpointer	data);

/* deprecated functions */

void       st_handler_set_info			(STHandler	*handler,
						 const char	*label,
						 const char	*copyright);
void       st_handler_set_icon			(STHandler	*handler,
						 int		size,
						 const guint8	*data);

#endif /* _ST_HANDLER_H */
